package networks;



import java.util.*;
import java.lang.*;
import java.io.*;
import java.net.*;
import javax.swing.text.*;
import javax.swing.text.html.*;



import parser.*;

/**
 * Wrapper class for the Google Search Engine
 */

 public class GoogleWrapper extends SearchWrapper{

	
	public final static java.lang.String GOOGLEQUERYSTRING = "http://www.google.com/search?";
	private int _maxReturnCount = 10;
	private Vector _searchWordList = null;




	public final static java.lang.String ADVANCEDSEARCH_QUERY_STRING = "as_q=";
	public final static java.lang.String ADVANCEDSEARCH_EXACT_QUERY_STRING = "&as_epq=";
	public final static java.lang.String ADVANCEDSEARCH_WITHOUT_WORD_STRING = "&as_eq=";
	public final static java.lang.String ADVANCEDSEARCH_ANYWORD_STRING = "&as_oq=";
	public final static java.lang.String ADVANCEDSEARCH_NUMRESULTS_STRING = "&num=";
	public final static java.lang.String ADVANCEDSEARCH_LANGUAGE_STRING= "&lr=";
	public final static java.lang.String ADVANCEDSEARCH_DATE_STRING= "&as_qdr=all";
	public final static java.lang.String ADVANCEDSEARCH_OCCURRENCELOC_STRING= "&as_occt=any";
	public final static java.lang.String ADVANCEDSEARCH_DOMAINFILTER_STRING= "&as_dt=i";
	public final static java.lang.String ADVANCEDSEARCH_SITESEARCH_STRING= "&as_sitesearch=";
	public final static java.lang.String ADVANCEDSEARCH_FILTEROPTION_STRING= "&safe=off";
	public final static java.lang.String ADVANCEDSEARCH_SETSEARCHPARAM_STRING= "&btnG=Google+Search";
	public final static java.lang.String REGULARSEARCH_QUERY_STRING = "q=";
	
	
	
	
/**
 * GoogleWrapper constructor comment.
 */
public GoogleWrapper() {
	super();
}

/**
 * GoogleWrapper constructor comment.
 */
public GoogleWrapper(int n) {
	super();

	set_maxResults(n);

	
}

/**
 * Used to add a search term before performQuery method is called
 */
public void addSearchWord(String word) {

	get_searchWordList().add(word);

}
/**
 * Clears the search word list for a new query
 */
public void clearSearchWordList() {

	get_searchWordList().clear();

}


	








/**
 * Insert the method's description here.
 * Creation date: (8/8/01 11:17:08 PM)
 * @return java.util.Vector
 */
public java.util.Vector get_searchWordList() {

	if(_searchWordList == null) {
		_searchWordList = new Vector();
	}
	
	return _searchWordList;
}


/**
 * Insert the method's description here.
 * 
 * @param args java.lang.String[]
 */
public static void main(String[] args) {
	
	
	GoogleWrapper a = new GoogleWrapper(8);

	a.addSearchWord("body");
	a.addSearchWord("solutions");
	SearchResultVector sr =	a.performQuery();

	for(int i=0; i < sr.size(); i++) {
		SearchResult s = (SearchResult) sr.elementAt(i);
		System.out.println(s.getURL().toString() + "  "+ s.getTitle());
	}
	

	
	System.exit(0);
	
	
	
	}







	public final static java.lang.String ADVANCEDSEARCH_BACKLINKPARAM_STRING= "as_lq=";
	public final static java.lang.String ADVANCEDSEARCH_SIMILARPAGEPARAM_STRING= "as_rq=";









/**
 * Extracts URLs from a query
 * Creation date: (7/24/01 1:20:57 AM)
 * @return SearchResultVector
 */
private SearchResultVector generateSearchResultInfo(HTMLDocument htmlDoc) {

	SearchResultVector searchResultList = new SearchResultVector();

	// Used to keep track of redundant URLs
	HashMap URL_list = new HashMap();
	HTMLDocumentParser p = new HTMLDocumentParser();
	p.setHTMLDoc(htmlDoc);

	String strTemp = null;
	String title = null;

	if (htmlDoc == null)
		return searchResultList;

	try {

		ElementIterator iterator = new ElementIterator(htmlDoc);
		Element elem;
		while ((elem = iterator.next()) != null) {

			SimpleAttributeSet sas = 
				(SimpleAttributeSet) elem.getAttributes().getAttribute(HTML.Tag.A); 

			if (sas != null) {
				if (sas.getAttribute(HTML.Attribute.HREF) != null) {
					strTemp = sas.getAttribute(HTML.Attribute.HREF).toString();
					// Filter out URLs within Google domain
					if (!(strTemp.startsWith("/")
						|| strTemp.startsWith("http://directory.google.com")
						|| strTemp.startsWith("http://www.google.com")
						|| strTemp.startsWith("http://images.google.com")
						|| strTemp.startsWith("http://groups.google.com")
						|| strTemp.startsWith("http://translate.google.com")
						|| strTemp.startsWith("http://toolbar.google.com"))) {

						//title = p.getTextFromElement(elem);
						int startOffset = elem.getStartOffset();
						while ((elem = iterator.next()) != null)
							if (elem.getName().equals(HTML.Tag.BR.toString()))
								break;
						if (elem == null)
							break;
						int length = elem.getEndOffset() - startOffset;
						title= htmlDoc.getText(startOffset, length);
						// Check for redundancy
						if (!(URL_list.containsKey(strTemp))) {

							URL_list.put(strTemp, strTemp);
							searchResultList.add(
								new SearchResult(new URL(strTemp.toString()), title, strTemp.toString())); 

						}
					}
				}
			}
		}

	} catch (Exception e) {
		System.out.println(e.toString() + " in generateSearchResultInfo(HTMLDocument)"); 
	}

	return searchResultList;
}





/**
 * Performs Google's InwardLink Query
 * Creation date: (8/8/01 8:57:25 PM)
 * @return util.SearchResult[]
 */
public SearchResultVector performBackLinkQuery(String urlStr) {

	StringBuffer sbURL = new StringBuffer();
	SearchResultVector sr = new SearchResultVector();

	HTMLDocument hdoc = null;

	// return an empty search result vector if no URL is specified	
	if(urlStr == null) return sr;

	// generate query string
	sbURL.append(GOOGLEQUERYSTRING + ADVANCEDSEARCH_BACKLINKPARAM_STRING + urlStr + ADVANCEDSEARCH_NUMRESULTS_STRING+ get_numMaxResults());

	try {


		URL url = new java.net.URL(sbURL.toString());

		hdoc = sendQuery(url);

		if(hdoc == null) return sr;

		sr = generateSearchResultInfo(hdoc);


	
	} catch (Exception e) {
		System.out.println(e + " in performBackLinkQuery(String)");
	}

	return sr;
}

/**
 * Performs a regular search query.
 *
 * Search terms must be added to the searchWordList
 * by use of the addSearchWord(String) method.
 */
public SearchResultVector performQuery() {

	StringBuffer sbURL = new StringBuffer(GOOGLEQUERYSTRING+REGULARSEARCH_QUERY_STRING + generateQueryString() + "&n="+ get_numMaxResults());
	SearchResultVector sr = new SearchResultVector();
	
	HTMLDocument hdoc = null;
	
	if(get_searchWordList().size() == 0) return sr;

	try {

		URL url = new java.net.URL(sbURL.toString());

		hdoc = sendQuery(url);
	
		sr = generateSearchResultInfo(hdoc);
	
	} catch (Exception e) {
		System.out.println(e + " in performQuery()");
	}

	return sr;

}

/**
 * Performs Google's Similar Page Query
 * Creation date: (8/8/01 8:57:25 PM)
 * @param String urlStr (URL to find similar pages for)
 * @return SearchResultVector
 */
public SearchResultVector performSimilarPageQuery(String urlStr) {

	StringBuffer sbURL = new StringBuffer();
	SearchResultVector sr = new SearchResultVector();

	HTMLDocument hdoc = null;

	// return an empty search result vector if no URL is specified	
	if(urlStr == null) return sr;

	// generate query string
	sbURL.append(GOOGLEQUERYSTRING + ADVANCEDSEARCH_SIMILARPAGEPARAM_STRING + urlStr + ADVANCEDSEARCH_NUMRESULTS_STRING + get_numMaxResults() );

	try {

		
		URL url = new java.net.URL(sbURL.toString());
		hdoc = sendQuery(url);

		sr = generateSearchResultInfo(hdoc);
	
	} catch (Exception e) {
		System.out.println(e + " in performSimilarPageQuery()");
	}

	return sr;
}



/**
 * Insert the method's description here.
 * Creation date: (8/8/01 11:39:37 PM)
 * @return int
 */
public int get_numMaxResults() {
	return _maxReturnCount;
}

/**
 * Sets the maximum number of search terms to return
 * Creation date: (8/8/01 11:39:37 PM)
 * @param new_numResults int
 */
public void set_maxResults(int new_numResults) {
	_maxReturnCount = new_numResults;
}
}